<?php

class MPOSRequest
{
    private $commerce_token = '';
    private $service_token = '';
    private $service_token_invoice = '';
    private $id_money = 1;

    /**
     * MPOS Request Constructor
     */
    function __construct($config) {
        $this->commerce_token = $config['commerce_token'];
        $this->service_token = $config['service_token'];
        $this->service_token_invoice = $config['service_token_invoice'];
//         $this->id_money = $config['id_money'];
    }

    /**
     * Validate expiration date
     */
    function validateExpiryDate($expiry_date) {
        $expiry_date_format = $this->getExpiryFormat($expiry_date);
        $is_valid = false;

        if( $expiry_date_format['YY'] >= date('Y') ) {
            if ( ($expiry_date_format['YY'] == date('Y') && $expiry_date_format['MM'] >= date('m')) || $expiry_date_format['YY'] > date('Y') ) {
                $is_valid = true;
            }
        }

        return $is_valid;
    }
    
    /**
     * Get MPOS Object
     */
    function getMPOSRequestStruct($request, $order) {
        global $woocommerce;

        $challenge = $this->getChallenge($request);
        $currency_code = $this->getCurrencyCode(get_woocommerce_currency());
        $expiry_date_format = $this->getExpiryFormat(str_replace(' ', '', $request['wc_mpos_payment_gateway-card-expiry']));
        $card_holder = explode(" ", $request['wc_mpos_payment_gateway-card-holder']);
        $card_number = str_replace(' ', '', $request['wc_mpos_payment_gateway-card-number']);
        $firstName = $card_holder[0];
+       $lastName = str_replace($firstName.' ', '', $request['wc_mpos_payment_gateway-card-holder']);
        
        $data = array(
            "comToken" => $this->commerce_token,
            "challenge" => $challenge,
            "ccPayInfo" => [
                "amount" => $order->get_total(),
                "tax" => 0,
                "cardNumber" => $card_number,
                "cardExpDate" => substr($expiry_date_format['YY'], -2).$expiry_date_format['MM'],
                "cardCvv" => $request['wc_mpos_payment_gateway-card-cvc'],
                "guidBric" => "",
                "currencyCode" => $currency_code,
                "numTranf" => $order->id,
                "firstName" => $firstName,
                "lastName" => $lastName,
                "address" => $request['billing_address_1'],
                "city" => $request['billing_city'],
                "state" => $request['billing_state'],
                "zipCode" => $request['billing_postcode'],
                "descriptor" => get_bloginfo('name')
            ],
            "processDate" => "",
            "email" => $request['billing_email'],
            "description" => "Online shopping at " . get_bloginfo('name'),
            "returnUrlString" => "",
            "additionalParamsUrlString" => "",
            "lang" => "ES"
        );

        return wp_json_encode($data);
    }

    /**
     * Get MPOS Object
     */
    function getMPOSInvoiceRequestStruct($request, $order, $trxRef, $operation_code, $description) {
        global $woocommerce;

        $challenge = $this->getChallengeInvoice($request);

        $date_modified = $order->get_date_modified();
        $date_paid = $order->get_date_paid();
        $date = empty( $date_paid ) ? $date_modified : $date_paid;
        $dateFormat = date_format($date, 'Y-m-d\TH:i:s');

        $amount = 0.0;
        $discount = 0.0;
        $subtotal = 0.0;
        $tax = $order->get_total_tax();
        $total = 0.0;

        foreach ( $order->get_items() as $item_id => $item ) {
            $item_data = $item->get_data();
            $amount += $item_data['total'];
            wc_add_notice(  $item_data  , 'error');
        }

        foreach ( $order->get_fees() as $item_id => $item ) {
            $item_data = $item->get_data();
            if(strpos($item_data['name'], 'Descuento') !== false){
                $discount += $item_data['total'];
            }
            wc_add_notice(  $item_data  , 'error');
        }
        $discount *= -1;

        //$description = "Online shopping at @sitename - Order (@order) - @reference";
        $description = str_replace("@sitename", get_bloginfo('name'), $description);
        $description = str_replace("@order", $order->id . '', $description);
        $description = str_replace("@reference", $trxRef, $description);

        $data = array(
            "challenge" => $challenge,
            "payload" => [
                "name" => $request['wc_mpos_payment_gateway-card-holder'],
                "email" => $request['billing_email'],
                "idMoney" => $this->id_money,
                "phone" => $request['billing_phone'],
                "dtBilled" => $dateFormat,
                "dtPaid" => $dateFormat,
                "dtCreditExpire" => $dateFormat,
                "codOper" => $operation_code,
                "location" => '0,0',
                "description" => $description,
                "amount" => $amount,
                "discount" => $discount,
                "subtotal" => $amount - $discount,
                "tip" => 0.0,
                "doTax" => true,
                "totTax" => $order->get_total() - ($amount - $discount),
                "total" => $order->get_total(),
                "totalPaid" => $order->get_total()
            ]
        );

        return wp_json_encode($data);
    }

    /**
     * Get currency code by string currency
     */
    function getCurrencyCode($currency) {
        // Note: Add codes as required.
        $currency_codes = array (
            '840' => 'USD',
            '558' => 'NIO',
            '170' => 'COP',
        );
        $currency_code = array_search ($currency, $currency_codes);
        return $currency_code;
    }

    /**
     * Get expiry date format
     */
    function getExpiryFormat($card_expiry) {
        $expiry_date = explode("/", $card_expiry);
        $expiryYear = $expiry_date[1];
        if(strlen($expiryYear) == 2){
            $expiryYear = '20'.$expiryYear;
        }

        $expiry_date_format = array (
            'MM' => $expiry_date[0],
            'YY' => $expiryYear,
        );
        return $expiry_date_format;
    }

    /**
     * Get challenge
     */
    function getChallenge($request) {
        $challenge_data = $this->commerce_token . '-' . str_replace(' ', '', $request['wc_mpos_payment_gateway-card-number']) . '-' . $request['billing_email'] . '-' . $this->service_token;
        $challenge = hash('sha512', $challenge_data);
        return $challenge;
    }

    /**
     * Get challenge invoice
     */
    function getChallengeInvoice($request) {
        $challenge_data = $this->service_token_invoice . "-Invoice";
        $challenge = hash('sha512', $challenge_data);
        return $challenge;
    }
}