<?php
/**
 * Plugin Name: MPOS Global Payment Gateway
 * Plugin URI: https://mpos.co
 * Description: mPos Global es una pasarela de pagos segura, certificada PCI y avalada por las marcas, disponible para america latina.
 * Author: MPOS Global
 * Author URI: https://www.mpos.co
 * Version: 1.6.0
 * License: GPL2
 * Text Domain: wc_mpos_payment_gateway
 * Domain Path: /i18n/languages/
 *
 * Copyright: (c) 2020 MPOS Global
 *
 * @package   wc_mpos_payment_gateway
 * @author    MPOS Global
 * @category  Admin
 * @copyright Copyright (c) 2020 MPOS Global
 */

require "app/MPOSRequest.php";

defined( 'ABSPATH' ) or exit;

// Make sure WooCommerce is active
if ( ! in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
  return;
}

/**
 * Add the gateway to WC Available Gateways
 * 
 * @since 1.0.0
 * @param array $gateways all available WC gateways
 * @return array $gateways all WC gateways
 */
function wc_mpos_global_add_to_gateways($gateways) {
  $gateways[] = 'wc_mpos_payment_gateway';
  return $gateways;
}
add_filter( 'woocommerce_payment_gateways', 'wc_mpos_global_add_to_gateways' );

/**
 * Adds plugin page links
 * 
 * @since 1.0.0
 * @param array $links all plugin links
 * @return array $links all plugin links + our custom links (i.e., "Settings")
 */
function wc_mpos_payment_gateway_plugin_links($links) {
  $plugin_links = array(
    '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=checkout&section=wc_mpos_payment_gateway' ) . '">' . __( 'Configure', 'wc_mpos_payment_gateway' ) . '</a>'
  );

  return array_merge($plugin_links, $links);
}
add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'wc_mpos_payment_gateway_plugin_links' );


// mPos Global tab in product woocommerce.

// Add a custom product setting tab to edit product pages options FOR SIMPLE PRODUCTS only
add_filter( 'woocommerce_product_data_tabs', 'mpos_global_tab', 50, 1 );
function mpos_global_tab( $tabs ) {
    $tabs['discount'] = array(
        'label' => __( 'Mpos Global', 'woocommerce' ),
        'target' => 'mpos_global_data', // <== to be used in the <div> class of the content
        'class' => array('show_if_simple'), // or 'hide_if_simple' or 'show_if_variable'…
    );

    return $tabs;
}

// Add/display custom Fields in the custom product settings tab
add_action( 'woocommerce_product_data_panels', 'mpos_global_tab_content', 10 );
function mpos_global_tab_content() {
    global $post;

    echo '<div id="mpos_global_data" class="panel woocommerce_options_panel">'; // <== Here we use the target attribute

    woocommerce_wp_checkbox(  array(
    'id'            => '_apply_global_bank_discount', 
    'wrapper_class' => 'show_if_simple', 
    'label'         => __( 'Descuento Global Bank', 'woocommerce' ),
    'description'   => __( 'Aplica descuento Global Bank', 'woocommerce' ),
    'default'     => '0',
    'desc_tip'      => false,
    ) );

    echo '</div>';
}

// Save the data value from the custom fields for simple products
add_action( 'woocommerce_process_product_meta_simple', 'save_mpos_global_tab_content', 50, 1 );
function save_mpos_global_tab_content( $post_id ) {
    $boolean_field = $_POST['_apply_global_bank_discount'];
    update_post_meta( $post_id, '_apply_global_bank_discount', esc_attr( $boolean_field ) );
}

/**
 * MPOS Payment Gateway
 *
 * Provides an MPOS Payment Gateway.
 *
 * @class     WC_MPOS_Payment_Gateway
 * @extends   WC_Payment_Gateway
 * @version   1.0.0
 * @package   WooCommerce/Classes/Payment
 * @author    MPOS Global
 */
add_action( 'plugins_loaded', 'wc_mpos_payment_gateway_init', 11 );

function wc_mpos_payment_gateway_init() {
  class WC_MPOS_Payment_Gateway extends WC_Payment_Gateway {

    private $REQUEST_URL = 'https://secure.mposglobal.com/RestEngine/clickPos';
    private $REQUEST_URL_AUTH = 'https://secure.mposglobal.com/RestEngine/clickPos/auth';
    private $INVOICE_REQUEST_URL = 'https://secure.mposglobal.com/RestEngine/';
    private $AUTHORIZATION = 'lCVcDu3UfVGfRuK5MsetZgJGpwkc2vQn';

    private $IS_DEBUG;

    /**
     * Gateway Constructor
     */
    public function __construct() {
      $this->id                 = 'wc_mpos_payment_gateway';
      $this->icon               = plugin_dir_url( __FILE__ ) . 'img/mpos-global-brands.png';
      $this->has_fields         = true;
      $this->method_title       = __( 'MPOS Global Payment', 'wc_mpos_payment_gateway' );
      $this->method_description = __( ' mPos Global es una pasarela de pagos segura, certificada PCI y avalada por las marcas, disponible para america latina. <br>      
      Solicita tu token! <a href="https://mpos.co/" target="_blank">MPOS Global</a> <img src="https://mpos.co/thirdpassets/logoPCI.png" title="" height="40" />', 'wc_mpos_payment_gateway' );
     
      $this->supports = array('products');
     
      $this->init_form_fields();
     
      $this->init_settings();
      $this->title = $this->get_option('title');
      $this->enabled = $this->get_option('enabled');
     
      add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
      add_action( 'wp_enqueue_scripts', array( $this, 'wc_mpos_scripts' ) );
      add_action( 'woocommerce_api_mpos_payment_gateway', array( $this, 'webhook' ) );

      $this->IS_DEBUG = false;
    }
 
    /**
     * Initialize Gateway Settings Form Fields
     */
    public function init_form_fields() {
      $this->form_fields = apply_filters( 'wc_mpos_payment_gateway_form_fields', array(
        'enabled' => array(
          'title'   => __( 'Habilitar / Deshabilitar', 'wc_mpos_payment_gateway' ),
          'type'    => 'checkbox',
          'label'   => __( 'Habilitar portal de pagos MPOS Global.', 'wc_mpos_payment_gateway' ),
          'default' => 'yes'
        ),
        'title' => array(
          'title'       => __( 'Título', 'wc-gateway-offline' ),
          'type'        => 'text',
          'default'     => __( 'Pago con MPOS Global', 'wc-gateway-offline' ),
          'desc_tip'    => true,
        ),
        'commerce_token' => array(
          'title'       => __( 'Token del comercio', 'wc_mpos_payment_gateway' ),
          'type'        => 'text',
          'description' => __( 'Este token es proporcionado por MPOS Global.', 'wc_mpos_payment_gateway' ),
          'desc_tip'    => true,
        ),
        'service_token' => array(
          'title'       => __( 'Token del servicio', 'wc_mpos_payment_gateway' ),
          'type'        => 'text',
          'description' => __( 'Este token es proporcionado por MPOS Global.', 'wc_mpos_payment_gateway' ),
          'desc_tip'    => true,
        ),
        'amount' => array(
          'title'       => __( 'Monto mínimo', 'wc_mpos_payment_gateway' ),
          'type'        => 'text',
          'description' => __( 'El monto mínimo que desea cobrar.', 'wc_mpos_payment_gateway' ),
          'desc_tip'    => true,
        ),
//         'id_money' => array(
//           'title'       => __( '# moneda', 'wc_mpos_payment_gateway' ),
//           'type'        => 'text',
//           'description' => __( 'El ID de la moneda dentro de MPOS Global.', 'wc_mpos_payment_gateway' ),
//           'desc_tip'    => true,
//           'default' => '1'
//         ),
        'enable_redirect' => array(
          'title'   => __( 'Habilitar / Deshabilitar', 'wc_mpos_payment_gateway' ),
          'type'    => 'checkbox',
          'label'   => __( 'Habilitar la redirección despues de un pago exitoso.', 'wc_mpos_payment_gateway' ),
          'default' => 'true'
        ),
        'url_to_redirect' => array(
          'title'       => __( 'URL a redireccionar', 'wc_mpos_payment_gateway' ),
          'type'        => 'text',
          'description' => __( 'Dirección URL a redireccionar después de realizar pagos exitosamente.', 'wc_mpos_payment_gateway' ),
          'desc_tip'    => true,
        ),
        'send_invoice' => array(
          'title'   => __( 'Habilitar / Deshabilitar', 'wc_mpos_payment_gateway' ),
          'type'    => 'checkbox',
          'label'   => __( 'Habilitar la creación/envío de factura.', 'wc_mpos_payment_gateway' ),
          'default' => 'true'
        ),
        'service_token_invoice' => array(
          'title'       => __( 'Token del servicio para facturación', 'wc_mpos_payment_gateway' ),
          'type'        => 'text',
          'description' => __( 'Este token es proporcionado por MPOS Global.', 'wc_mpos_payment_gateway' ),
          'desc_tip'    => true,
        ),
        'invoice_message' => array(
          'title'       => __( 'Descripción de factura', 'wc_mpos_payment_gateway' ),
          'type'        => 'textarea',
          'description' => __( 'Descripción de la factura. <br> 1) @sitename => Nombre del sitio. <br> 2) @order => Número de orden. <br> 3) @reference => Referencia de la transacción.', 'wc_mpos_payment_gateway' ),
        //   'desc_tip'    => true,
          'default' => "Compra online en @sitename - Orden Nº @order - Referencia: @reference"
        ),
        'enable_discount_global_bank' => array(
          'title'   => __( 'Habilitar / Deshabilitar', 'wc_mpos_payment_gateway' ),
          'type'    => 'checkbox',
          'label'   => __( 'Habilitar descuento al procesar .', 'wc_mpos_payment_gateway' ),
          'default' => 'no'
        ),
        'discount' => array(
          'title'       => __( 'Descuento (%)', 'wc_mpos_payment_gateway' ),
          'type'        => 'text',
          'description' => __( 'El descuento en porcentaje que desea aplicar.', 'wc_mpos_payment_gateway' ),
          'desc_tip'    => true,
          'default' => "0.0"
        ),
//         'global_bank_bins' => array(
//           'title'       => __( 'Global Bank BINs', 'wc_mpos_payment_gateway' ),
//           'type'        => 'textarea',
//           'description' => __( 'Lista de BINs separadas por coma (,).', 'wc_mpos_payment_gateway' ),
//           'desc_tip'    => true,
//           'default' => "494148, 494171, 416833, 527610, 434985"
//         ),
//         'global_bank_card_detected_message' => array(
//           'title'       => __( 'Mensaje de tarjeta con Descuento', 'wc_mpos_payment_gateway' ),
//           'type'        => 'textarea',
//           'description' => __( 'Mensaje de tarjeta con descuento detectada.', 'wc_mpos_payment_gateway' ),
//           'desc_tip'    => true,
//           'default' => "Tarjeta de global bank detectada, se aplicará un descuento de ser el caso."
//         ),
        'transaction_denied_message' => array(
          'title'       => __( 'Transacción denegada', 'wc_mpos_payment_gateway' ),
          'type'        => 'textarea',
          'description' => __( 'Mensaje de transacción denegada.', 'wc_mpos_payment_gateway' ),
          'desc_tip'    => true,
          'default' => "Transacción denegada."
        ),
      ) );
    }
 
    /**
     * Credit Card Form
     */
    public function payment_fields() {      
      $this->description = 'Pague con tranquilidad! Este formulario de pago es seguro y cumple con los estándares más altos de seguridad a nivel internacional. mPos Global es una pasarela de pagos certificada PCI. <img src="https://mpos.co/thirdpassets/logoPCI.png" title="" height="40" />';
      
      $this->init_settings();
      $config = $this->settings;

      echo wpautop(wp_kses_post($this->description));

      echo '<div id="mg-enable-discount-global-bank" style="display:none">' . $config["enable_discount_global_bank"] . '</div>';
      echo '<div id="mg-discount" style="display:none">' . $config["discount"] . '</div>';
//       echo '<div id="mg-global-bank-bins" style="display:none">' . $config["global_bank_bins"] . '</div>';
      echo '<div id="mg-global-bank-bins" style="display:none"></div>';
     echo '<div id="mg-global-bank-card-detected-message" style="display:none"> </div>';
//       echo '<div id="mg-global-bank-card-detected-message" style="display:none">' . $config["global_bank_card_detected_message"] . '</div>';

      add_filter( 'woocommerce_credit_card_form_fields', 'add_card_holder', 10, 2 );
      function add_card_holder( $default_fields, $id ){
        $cc_fields = array(
          'name-on-card' => 
          '          
          <div id="ccard-wrapper"></div>
          <input id="' . esc_attr( $id ) . '-card-is-valid" class="input-text wc-credit-card-form-card-is-valid" type="hidden" name="' . $id . '-card-is-valid" />
          
          <p class="form-row form-row-wide">
            <label for="' . esc_attr( $id ) . '-card-holder">' . __( 'Titular de la tarjeta', 'woocommerce' ) . ' <span class="required">*</span></label>
            <input id="' . esc_attr( $id ) . '-card-holder" class="input-text wc-credit-card-form-card-holder" type="text" maxlength="36" autocomplete="off" placeholder="" name="' . $id . '-card-holder' . '" />
          </p>
          <p class="form-row form-row-wide woocommerce-validated">
            <label for="' . esc_attr( $id ) . '-card-number">' . __( 'Número de tarjeta', 'woocommerce' ) . ' <span class="required">*</span></label>
            <input id="' . esc_attr( $id ) . '-card-number" class="input-text wc-credit-card-form-card-number" inputmode="numeric" autocomplete="cc-number" autocorrect="no" autocapitalize="no" spellcheck="no" type="tel" placeholder="•••• •••• •••• ••••" name="' . $id . '-card-number">
          </p>
          <p class="form-row form-row-first woocommerce-validated">
            <label for="' . esc_attr( $id ) . '-card-expiry">' . __( 'Fecha de expiración (MM/YY)', 'woocommerce' ) . ' <span class="required">*</span></label>
            <input id="' . esc_attr( $id ) . '-card-expiry" class="input-text wc-credit-card-form-card-expiry" inputmode="numeric" autocomplete="cc-exp" autocorrect="no" autocapitalize="no" spellcheck="no" maxlength="7" type="tel" placeholder="MM / YY" name="' . $id . '-card-expiry">
          </p>
          <p class="form-row form-row-last woocommerce-validated">
            <label for="' . esc_attr( $id ) . '-card-cvc">' . __( 'CVV', 'woocommerce' ) . ' <span class="required">*</span></label>
            <input id="' . esc_attr( $id ) . '-card-cvc" class="input-text wc-credit-card-form-card-cvc" inputmode="numeric" autocomplete="off" autocorrect="no" autocapitalize="no" spellcheck="no" type="tel" maxlength="3" placeholder="CVV" name="' . $id . '-card-cvc" style="width:100px">
          </p>
          <div class="clear"></div>
          '
        );
        return $cc_fields;
      }

      $cc_form           = new WC_Payment_Gateway_CC();
      $cc_form->id       = $this->id;
      $cc_form->supports = $this->supports;
      $cc_form->form();
    }
 
    /*
     * MPOS resource requirements
     */
    public function wc_mpos_scripts(){
      // wp_enqueue_script('wc_mpos_scripts', plugin_dir_url( __FILE__ ) . 'components/card-master/dist/card.js', array( 'jquery' ), '', true);
      // wp_enqueue_style('wc_mpos_styles_main', plugin_dir_url( __FILE__ ) . 'components/card-master/dist/card.css');
      wp_enqueue_style('wc_mpos_styles_main_plugin', plugin_dir_url( __FILE__ ) . 'css/wc_mpos.css');
      wp_enqueue_script('wc_mpos_scripts_main', plugin_dir_url( __FILE__ ) . 'js/wc_mpos.js', array('jquery'));
    }

    /*
     * MPOS credit card validation
     */
    public function action_invalid_credit_card(){
      wc_add_notice('Tarjeta de crédito inválida.', 'error');
    }
 
    /*
     * Fields validation
     */
    public function validate_fields(){
      $status = true;

      if(empty($_POST['wc_mpos_payment_gateway-card-holder'])) {
        wc_add_notice('El titular de la tarjeta es requerido!', 'error');
        $status = false;
      }
      if(empty($_POST['wc_mpos_payment_gateway-card-number'])) {
        wc_add_notice('El número de tarjeta es requerido!', 'error');
        $status = false;
      }
      if(empty($_POST['wc_mpos_payment_gateway-card-expiry'])) {
        wc_add_notice('La fecha de expiración es requerida!', 'error');
        $status = false;
      }
      if(empty($_POST['wc_mpos_payment_gateway-card-cvc'])) {
        wc_add_notice('El código de seguridad (CVV) es requerido!', 'error');
        $status = false;
      }

      return $status;
    }

    /*
     * Processing Payment
     */
    public function process_payment($order_id) {
      global $woocommerce;

      $this->init_settings();
      $config = $this->settings;
      $mpos_instance = new MPOSRequest($config);

      $request = $_REQUEST;
      $order = wc_get_order($order_id);
      $card_expiry = str_replace(' ', '', $request['wc_mpos_payment_gateway-card-expiry']);

//       if($request['wc_mpos_payment_gateway-card-is-valid'] == 'true') {
        if($mpos_instance->validateExpiryDate($card_expiry) === true) {

          $card_number = str_replace(' ', '', $request['wc_mpos_payment_gateway-card-number']);
          $request_url = $this->REQUEST_URL;

          if($config['enable_discount_global_bank']==='yes'){
            $discount_percentage = number_format($config['discount'], 2);
            $title = "Descuento (Global Bank) - O" . $order_id;

            foreach($woocommerce->cart->get_coupons() as $code => $coupon) {
              if (strpos(strtolower($code), strtolower('Descuento (Global Bank)')) !== false) {
                $woocommerce->cart->remove_coupon( $code );
                $order->remove_coupon( $code );
              }
            }

            $is_global_bank_bins = false;
//             if (wc_is_global_bank_bins(explode(', ', $config['global_bank_bins']), $card_number)) {
//               $is_global_bank_bins =  true;
//             } 

            if($is_global_bank_bins){
              $items = $woocommerce->cart->get_cart();
              foreach($items as $item => $values) { 
                $product =  wc_get_product($values['data']->get_id()); 
                $name =  $product->get_title();
                $quantity =  $values['quantity'];
                $price_excluding_tax = wc_get_price_excluding_tax( $product ); 
                //$price_including_tax = wc_get_price_including_tax( $product ); 
        
                $apply_global_bank_discount = false;
                $postmetas = get_post_meta($values['data']->get_id());
                foreach($postmetas as $meta_key=>$meta_value) {
                  if($meta_key === "_apply_global_bank_discount"){
                    $apply_global_bank_discount = ($meta_value[0] === "yes");
                  }
                }
                if($apply_global_bank_discount){
                  wc_cart_add_coupon($order_id, $product->get_id(), $title, $price_excluding_tax * ($discount_percentage/100));
                }
              } 
            }
          
          }

          $order = wc_get_order($order_id);
          $order->calculate_totals();
          $order->save();

          if(round($order->get_total(), 2) === 0.0){
           $request_url = $this->REQUEST_URL_AUTH;
          }
          
          $request_body = $mpos_instance->getMPOSRequestStruct($request, $order);
      
          $response = (object) wp_remote_post($request_url, array(
            'method'      => 'POST',
            'body'        => $request_body,
            'headers'     => [
              'Contdent-Type' => 'application/json',
              'Authorization' => $this->AUTHORIZATION
            ],
            'timeout'     => 90,
            'blocking'    => true,
            'httpversion' => '1.0',
            'sslverify'   => true,
            'data_format' => 'body'
          ));

          $response_data = json_decode($response->body);

          if($this->IS_DEBUG){
            wc_add_notice("=========================================", 'error');
            wc_add_notice($request_body, 'error');
            wc_add_notice("=========================================", 'error');
            wc_add_notice($request_url, 'error');
            wc_add_notice("=========================================", 'error');
          }
          
          if (is_wp_error($response)) {
            wc_add_notice('El servicio no está disponible, comuníquese con MPOS Global.', 'error');

          } elseif ($response_data->isProcessed == 1) {

            if($response_data->isApproved == 1) {
              $location = $this->get_return_url($order);
              $transaction_reference = $response_data->transaction->trxRef;
              $operation_code = $response_data->transaction->codOper;

              if ($config['send_invoice'] === 'yes') {
                $invoice_struct = $mpos_instance->getMPOSInvoiceRequestStruct($request, $order, $transaction_reference, $operation_code, $config['invoice_message']);
                $response = $this->send_invoice($config, $invoice_struct);
                if($this->IS_DEBUG){
                  wc_add_notice("=========================================", 'error');
                  wc_add_notice(json_encode($invoice_struct ), 'error');
                  wc_add_notice("=========================================", 'error');
                  wc_add_notice(json_encode($response ), 'error');
                  wc_add_notice("=========================================", 'error');
                }
              }

              if(!$this->IS_DEBUG){
                $order->payment_complete();
                $order->reduce_order_stock();
                $order->add_order_note( 'Su pedido se pagó con éxito', true );
                $woocommerce->cart->empty_cart();

                if ($config['enable_redirect'] === 'yes' && (empty($config['url_to_redirect']) === false)) {
                  $order_data = $this->get_order_data($request, $order->get_data());
                  $location = $this->create_route($config['url_to_redirect'], $order_data);
                  if($config['enable_post'] === 'yes') {
                    $this->external_endpoint($url_to_redirect, $request, $order);
                  }
                }

                return array(
                'result' => 'success',
                'redirect' => $location
                );
              }

            } else {
              wc_add_notice('<h3>'.$config['transaction_denied_message'].'</h3><br/>', 'error');
              wc_add_notice('<h3 style="color: #000;">Transacci&oacute;n Denegada</h3><br /><small style="color: #000;">('.$response_data->transaction->authDesc. ')</small><br />Su tarjeta ha sido denegada y el pago no fue procesado. Verifique si ingreso los datos de su tarjeta adecuadamente e intente de nuevo.<br />En caso de ser la segunda vez que ve este mensaje, favor intente con otra tarjeta.<br />Si a usted le parece que no deber&iacute;a ver este mensaje, p&oacute;ngase en contacto con su instituci&oacute;n Bancaria.<br />Gracias', 'error');
            }

          } else {
            wc_add_notice('Transacci&oacute;n no procesada ('.$response_data->service->description.')', 'error');
          }

        } else {
          wc_add_notice('La fecha de vencimiento de la tarjeta de crédito no es válida.', 'error');
        }
//       } else {
//         do_action('woocommerce_after_checkout_validation', 'action_invalid_credit_card', 0,0);
//       }
    }

    /*
     * Pending: If you want exec any action after complete the payment
     */
    public function webhook() {
      // Pending
    }

    /*
     * Return the order data
     */
    private function get_order_data($request, $order) {
      $card_type = (substr($request['wc_mpos_payment_gateway-card-number'], 0, 1) == 4) ? 'Visa' : 'Mastercard' ;
      $card_identifier = substr($request['wc_mpos_payment_gateway-card-number'], -4);

      $data = array(
        'order_number' => $order['id'],
        'total' => $order['total'],
        'email' => $order['billing']['email'],
        'card_type' => $card_type,
        'card_identifier' => $card_identifier
      );

      $params = http_build_query($data, '', '&');

      return $params;
    }

    /*
     * Return a valid route
     */
    private function create_route($url, $order_data) {
      $parse_url = parse_url($url);
      $url_to_redirect = '';

      if (isset($parse_url['host']) || isset($parse_url['path'])) {
        if (isset($parse_url['scheme'])) {
          $url_to_redirect .= $parse_url['scheme'] . '://';
        } else {
          $url_to_redirect .= 'http://';
        }

        if (isset($parse_url['host'])) {
          $url_to_redirect .= $parse_url['host'];
        }

        if (isset($parse_url['port'])) {
          $url_to_redirect .= ':' . $parse_url['port'];
        }

        if (isset($parse_url['path'])) {
          $url_to_redirect .= $parse_url['path'];
        }

        if (isset($parse_url['query'])) {
          $url_to_redirect .= '?' . $parse_url['query'];
          $url_to_redirect .= '&' . $order_data;
        } else {
          $url_to_redirect .= '?';
          $url_to_redirect .= $order_data;
        }

        if (isset($parse_url['fragment'])) {
          $url_to_redirect .= '#' . $parse_url['fragment'];
        }
      }

      return $url_to_redirect;
    }

    /*
     * Send invoice
     */
    private function send_invoice($config, $invoice_struct) {
      $URL = $this->INVOICE_REQUEST_URL . $config['commerce_token'] . '/Invoice';

      $response = (object) wp_remote_post($URL, array(
        'method'      => 'POST',
        'body'        => $invoice_struct,
        'headers'     => [
          'Content-Type' => 'application/json',
          'Authorization' => $this->AUTHORIZATION
        ],
        'timeout'     => 90,
        'blocking'    => true,
        'httpversion' => '1.0',
        'sslverify'   => true,
        'data_format' => 'body'
      ));

      return $response;
    }   

    /*
     * Send form to external endpoint
     */
    private function external_endpoint($endpoint, $request, $order) {
      $card_identifier = substr($request['wc_mpos_payment_gateway-card-number'], -4);
      $card_type = (substr($request['wc_mpos_payment_gateway-card-number'], 0, 1) == 4) ? 'Visa' : 'Mastercard' ;
      $request['card_identifier'] = $card_identifier;
      $request['card_type'] = $card_type;
      unset($request['wc_mpos_payment_gateway-card-number']);
      unset($request['wc_mpos_payment_gateway-card-expiry']);
      unset($request['wc_mpos_payment_gateway-card-cvc']);

      $data = array(
              "request" => $request,
              "order" => $order->get_data()
          );

      $response = wp_remote_post($endpoint, array(
        'method'      => 'POST',
        'body'        => wp_json_encode($data),
        'headers'     => [
          'Content-Type' => 'application/json'
        ],
        'timeout'     => 90,
        'httpversion' => '1.0',
        'blocking'    => false,
        'sslverify'   => false
      ));

      if (is_wp_error($response)) {
          $error_message = $response->get_error_message();
          wc_add_notice($error_message, 'error');
      }
    }

  }
}





































/* Add js at the footer */
function add_wc_mpos_js(){
  if ( is_checkout() ) {
    wp_enqueue_script( 'checkout_script', plugins_url( '/js/wc_mpos.js', __FILE__ ), '', '', false );
    $localize_script = array(
      'ajax_url' => admin_url( 'admin-ajax.php' )
    );
    wp_localize_script( 'checkout_script', 'apply_coupon', $localize_script );
  }
}
add_action( 'wp_footer', 'add_wc_mpos_js', 10 );

/* 
  Load ajax 
*/
function load_ajax() {
  if ( !is_user_logged_in() ){
      add_action( 'wp_ajax_nopriv_update_order_review', 'update_order_review' );
  } else{
      add_action( 'wp_ajax_update_order_review',        'update_order_review' );
  }
}
add_action( 'init', 'load_ajax' );

/* 
  Update order review 
*/
function update_order_review() {
  if ( ! $_POST  ) {
    return;
  }

  global $woocommerce;

  parse_str($_REQUEST['post_data'], $params);
  $card_number = $params['card_number'];
  $setting_enable_discount_global_bank = $params['setting_enable_discount_global_bank'];
  $setting_discount = floatval($params['setting_discount']);
  $setting_global_bank_bins = []; // $params['setting_global_bank_bins'];
  $order_id =  -1;

  if($setting_enable_discount_global_bank ==='yes'){
    $discount_percentage = number_format($setting_discount, 2);
    $title = "Descuento (Global Bank) - U";

    foreach($woocommerce->cart->get_coupons() as $code => $coupon) {
      if (strpos(strtolower($code), strtolower('Descuento (Global Bank)')) !== false) {
        $woocommerce->cart->remove_coupon( $code );
      }
    }

    $is_global_bank_bins = false;
//     if (wc_is_global_bank_bins(explode(',', $setting_global_bank_bins), $card_number)) {
//       $is_global_bank_bins = true;
//     } 

    if($is_global_bank_bins){
      $items = $woocommerce->cart->get_cart();
      foreach($items as $item => $values) { 
        $product =  wc_get_product($values['data']->get_id()); 
        $name =  $product->get_title();
        $quantity =  $values['quantity'];
        $price_excluding_tax = wc_get_price_excluding_tax( $product ); 
        //$price_including_tax = wc_get_price_including_tax( $product ); 

        $apply_global_bank_discount = false;
        $postmetas = get_post_meta($values['data']->get_id());
        foreach($postmetas as $meta_key=>$meta_value) {
          if($meta_key === "_apply_global_bank_discount"){
            $apply_global_bank_discount = ($meta_value[0] === "yes");
          }
        }
        if($apply_global_bank_discount){
          wc_cart_add_coupon($order_id, $product->get_id(), $title, $price_excluding_tax * ($discount_percentage/100));
        }
      } 
    }
  }

  wp_die();
}


/*
  * Global Bank validation.
*/
function wc_is_global_bank_bins($bins, $card_number) {
  foreach ($bins as $bin) {
    if (substr($card_number, 0, strlen($bin)) === $bin) {
      return true;
    }
  }
  return false;
}

/*
* Add coupon.
*/
function wc_cart_add_coupon( $order_id, $product_id, $title, $amount) {
  global $woocommerce;

  $coupon_code = $title . " - P" . $product_id;

  $args = array(
    'posts_per_page' => -1,
    'post_type'      => 'shop_coupon',
    'post_status'    => 'publish',
    "s" => $coupon_code
  );
  $coupons = get_posts($args);
  if ( ! empty( $coupons ) ) {
    foreach ( $coupons as $coupon ) {
      wp_trash_post( $coupon->ID );
    }
  }

  $coupon = array(
      'post_title' => $coupon_code,
      'post_content' => '',
      'post_status' => 'publish',
      'post_author' => 1,
      'post_type'     => 'shop_coupon'
  );    

  $new_coupon_id = wp_insert_post( $coupon );

  update_post_meta( $new_coupon_id, 'discount_type', 'fixed_product' ); // Type: fixed_cart, percent, fixed_product, percent_product.
  update_post_meta( $new_coupon_id, 'coupon_amount', $amount . '' );
  update_post_meta( $new_coupon_id, 'individual_use', 'no' );
  update_post_meta( $new_coupon_id, 'product_ids', $product_id . '');
  update_post_meta( $new_coupon_id, 'exclude_product_ids', '' );
  update_post_meta( $new_coupon_id, 'usage_limit', '' );
  update_post_meta( $new_coupon_id, 'expiry_date', '' );
  update_post_meta( $new_coupon_id, 'apply_before_tax', 'no' );
  update_post_meta( $new_coupon_id, 'free_shipping', 'yes' );
  
  $woocommerce->cart->add_discount( sanitize_text_field( $coupon_code ));

  if($order_id !== -1){
    $order = wc_get_order($order_id);
    $order->apply_coupon($coupon_code);
    $order->save();
  }
}


add_filter( 'woocommerce_cart_needs_payment', 'filter_cart_needs_payment', 10, 2 );
function filter_cart_needs_payment( $needs_payment, $cart  ) {
    return  true;
}
